#include "snippets.lua"


function init()

    toolname = "lightning_stick"

    RegisterTool(toolname, "Lightning Stick", "MOD/vox/" .. toolname .. ".vox")
	SetBool("game.tool." .. toolname .. ".enabled", true)

    --[[
        animations
            [i]
                state
                [j]
                    {time, pos, rot}
    ]]

    -- default pos = Vec(0.4, -0.8, -0.35), QuatEuler(-30, 0, 10)
    animations = {}
    animations[#animations + 1] = {
        state = 1,
        timeLeft = nil,
        name = "ignite",
        effect = {false, false, true, true},
        pos = {makeAnimation(0.3, Vec(-0.15, 0.3, -0.35), Vec(-50, 0, 0)),
               makeAnimation(0.6, Vec(0.0, 0.0, 0.0), Vec(0, 0, 0)),
               makeAnimation(0.3, Vec(0.15, -0.3, 0.35), Vec(50, 0, 0))}
    }

    animation = nil

    first = true

    tt = defaultTransform()

    balls = {}

end

function tick(dt)

    if animation ~= nil and isDoingEffect(animation) then
        SetBool("level.bombard.ignite", true)
    else
        SetBool("level.bombard.ignite", false)
    end

    if isToolInHand() then
        if InputPressed("usetool") then
            if animation == nil then
                tt = defaultTransform()
                animation = deepcopy(animations[1])
            end
        end

        if animation ~= nil then
            handleAnimation(dt)
        else
            tt = defaultTransform()
        end

        SetToolTransform(tt)
    end
end

function update(dt)
    if first then
        first = false
    end

    balls = FindBodies("", true)--FindBodies("bombardball", true)

    if isToolInHand() then
        
    end
end

function draw(dt)
    if isToolInHand() then
        
    end
end

function isDoingEffect(a)
    if a ~= nil then
        return (animation.effect[animation.state] ~= nil and animation.effect[animation.state] == true)
    end
end

function isToolInHand()
    return GetString("game.player.tool") == toolname
end

function defaultTransform()
    return Transform(Vec(0.4, -0.8, -0.35), QuatEuler(-30, 0, 10))
end

function debugTotalAnimationOffset(a)
    --local t = defaultTransform()
    local pos = Vec()
    --local rx, ry, rz = GetQuatEuler(t.rot)
    local rot = Vec()

    for i=1, #a.pos do
        pos = VecAdd(pos, a.pos[i].pos)
        rot = VecAdd(rot, a.pos[i].rot)
    end
    DebugPrint(vecToString(pos) .. " " .. vecToString(rot))
end

function makeAnimation(time, v, q)
	local t = {
        time = 0,
        pos = Vec(),
        rot = Vec()
    }
	t.time = time
	t.pos = deepcopy(v)--VecScale(v, 1 / time)
	for i=1, 3 do
		t.rot[i] = q[i]--q[i] * (1 / time)
	end
	return deepcopy(t)
end

function handleAnimation(dt)

    local a = getAnimation(animation)
    local pos = tt.pos
    local rot = tt.rot
    local ratio = dt / a.time

    if animation.timeLeft == nil then
        animation.timeLeft = a.time
    end

    local posAdd = VecScale(a.pos, ratio)
    local rawRot = VecScale(a.rot, ratio)
    local rotAdd = QuatEuler(rawRot[1], rawRot[2], rawRot[3])

    tt = Transform(VecAdd(pos, posAdd), QuatRotateQuat(rot, rotAdd))

    animation.timeLeft = animation.timeLeft - dt
    if animation.timeLeft <= 0 then
        animation.state = animation.state + 1
        if animation.state > #animation.pos then
            animation = nil
        else
            local a2 = getAnimation(animation)
            animation.timeLeft = a2.time
        end
    end
end

function getAnimation(a)
    return a.pos[a.state]
end

function sparks(pos)
	local radius = 0.005
	local life = 0.45
	local count = 30
	local drag = 0.2
	local gravity = -2.5
	local alpha = 0.9
	
	--Set up the particle state
	ParticleReset()
	ParticleType("plain")
	ParticleRadius(radius)
	ParticleAlpha(alpha, alpha, "constant", 0.1 / life, 0.5)	-- Ramp up fast, ramp down after 50%
	ParticleGravity(gravity * randFloat(0.7, 1.3))				-- Slightly randomized gravity looks better
	ParticleDrag(drag)
	ParticleTile(4)
	ParticleCollide(1)
    ParticleEmissive(0.8, 0.2)
	
	--Emit particles
	for i=1, count do
        local red = 0.96 + randFloat(-0.04, 0.04)
        local green = 0.91 + randFloat(-0.04, 0.04)
        local blue = 0.50 + randFloat(-0.4, 0.4)
        ParticleColor(red, green, blue, 0.9, 0.9, 0.9)			-- Animating color towards white
		p = VecAdd(pos, randVec(2 * radius))
		local v = randVec(randFloat(1.5, 2.5))
        if v[2] < 0 then
            v[2] = -v[2]
        end
	
		--Randomize lifetime
		local l = randFloat(life * 0.8, life * 1.2)

		SpawnParticle(p, v, l)
	end
end











































