#include "../snippets.lua"

function init()
    joint = FindJoint("pitch")
    jointLock = GetJointMovement(joint)
    speed = 80

    local barrelShape = FindShape("barrelShape")
    local barrelBody = FindBody("barrelBody")

    local barrelTrigger = FindTrigger("barrel")
    local t = GetTriggerTransform(barrelTrigger)
    bt = GetBodyTransform(barrelBody)
    barrel = {}
    barrel.trigger = barrelTrigger
    barrel.localTransform = TransformToLocalTransform(bt, t)
    barrel.body = barrelBody
    barrel.shape = barrelShape

    big = HasTag(barrelBody, "big")

    canon = {
        loaded = false,
        time = {
            value = 0,
            default = 2.5
        },
        ignited = false
    }

    meshSprite = LoadSprite("MOD/img/mesh.png")
    meshSize = 0.3
    if big then
        meshSize = meshSize * 2
    end

    sndBoom = LoadSound("MOD/snd/boom.ogg")
    sndIgnite = LoadLoop("MOD/snd/ignite.ogg")
end

function tick(dt)

    bt = GetBodyTransform(barrel.body)
    movement()

    if canon.ignited then
        canon.time.value = canon.time.value - dt
    end

    drawMesh()

    if GetBool("level.bombard.ignite") then
        local ct = GetPlayerCameraTransform()
        local dir = TransformToParentVec(ct, Vec(0, 0, -1))
        local hit, dist, normal, shape = QueryRaycast(ct.pos, dir, 3)
        if barrel.shape == shape then
            if canon.loaded then
                canon.ignited = true
            end
        end
    end

    if canon.time.value <= 0 and canon.ignited then
        fire()
        smoke()
    end

    canon.loaded = isLoaded()
    if canon.loaded and not canon.ignited then
        canon.time.value = canon.time.default
    end
    if not canon.loaded then
        canon.time.value = 0
    end
end

function movement()
    local bt = GetBodyTransform(barrel.body)
    local t  = TransformToParentTransform(bt, barrel.localTransform)
    local tFwd = TransformToParentVec(t, Vec(0, 0, -1))

    t.pos = VecAdd(t.pos, VecScale(tFwd, 0.15))
    SetTriggerTransform(barrel.trigger,t)

    if GetPlayerGrabBody() == barrel.body then 
        SetJointMotor(joint, 0, 0)
        jointLock = GetJointMovement(joint)
    else
        SetJointMotorTarget(joint, jointLock)
    end
end

function fire()
    local balls = FindBodies("", true)--FindBodies("bombardball", true)
    canon.ignited = false
    for i=1, #balls do
        local t = GetBodyTransform(balls[i])
        if IsPointInTrigger(barrel.trigger,t.pos) then
            PlaySound(sndBoom, t.pos, 0.6)
            SetBodyVelocity(balls[i], TransformToParentVec(GetBodyTransform(barrel.body), Vec(0, 0, -speed)))
            --return
        end
    end
    local t = GetPlayerTransform()
    if IsPointInTrigger(barrel.trigger,t.pos) then
        PlaySound(sndBoom, t.pos, 0.6)
        SetPlayerVelocity(TransformToParentVec(GetBodyTransform(barrel.body), Vec(0, 0, -speed)))
    end
end

function isLoaded()
    local balls = FindBodies("", true)--FindBodies("bombardball", true)
    for i=1, #balls do
        local t = GetBodyTransform(balls[i])
        if IsPointInTrigger(barrel.trigger,t.pos) then
            return true
        end
    end
    local t = GetPlayerTransform()
    if IsPointInTrigger(barrel.trigger,t.pos) then
        return true
    end
    return false
end

function drawSpriteLine(Pos1, Pos2, sprite)
    local length = VecLength(VecSub(Pos1, Pos2))
    local t = Transform(VecLerp(Pos1, Pos2, 0.5))
    local xAxis = VecNormalize(VecSub(Pos1, Pos2))
    local zAxis = VecNormalize(VecSub(Pos1, GetCameraTransform().pos))
    t.rot = QuatAlignXZ(xAxis, zAxis)
    DrawSprite(sprite, t, length, 0.05, 0.6, 0.45, 0.2, 1, true)
end

function getMeshPos()
    return Vec(0, 0.7 * meshSize * (canon.time.value / canon.time.default), 0.3 * meshSize * (canon.time.value / canon.time.default))
end

function drawMesh()
    local startPos = Vec(0.05, 0.9, 0.5)
    if big then
        startPos = VecScale(startPos, 2)
    end
    local a = TransformToParentPoint(bt, startPos)
    local b = TransformToParentPoint(bt, VecAdd(startPos, getMeshPos()))
    if canon.ignited then
        sparks(b)
        PlayLoop(sndIgnite, b, 0.4)
    end
    drawSpriteLine(a, b, meshSprite)
end

function sparks(pos)
	local radius = 0.005
	local life = 0.2
	local count = 30
	local drag = 0.2
	local gravity = -2.5
	local alpha = 0.9
	
	--Set up the particle state
	ParticleReset()
	ParticleType("plain")
	ParticleRadius(radius)
	ParticleAlpha(alpha, alpha, "constant", 0.1 / life, 0.5)	-- Ramp up fast, ramp down after 50%
	ParticleGravity(gravity * randFloat(0.7, 1.3))				-- Slightly randomized gravity looks better
	ParticleDrag(drag)
	ParticleTile(4)
	ParticleCollide(1)
    ParticleEmissive(0.8, 0.2)
	
	--Emit particles
	for i=1, count do
        local red = 0.96 + randFloat(-0.04, 0.04)
        local green = 0.88 + randFloat(-0.04, 0.04)
        local blue = 0.50 + randFloat(-0.4, 0.4)
        ParticleColor(red, green, blue, 0.8, 0.6, 0.2)			-- Animating color towards white
		p = VecAdd(pos, randVec(2 * radius))
		local v = randVec(randFloat(1.5, 2.5))
        if v[2] < 0 then
            v[2] = -v[2]
        end
        v[2] = v[2] + 0.3
        v = QuatRotateVec(bt.rot, QuatRotateVec(QuatEuler(55, 0, 0), v))
	
		--Randomize lifetime
		local l = randFloat(life * 0.8, life * 1.2)

		SpawnParticle(p, v, l)
	end
end

function smoke()
    local pos = bt.pos
    local radius = 0.5
	local life = 12
	local count = 700
	local drag = 0.2
	local gravity = 0
	local alpha = 0.9

    if big then
        radius = radius * 2
    end
	
	--Set up the particle state
	ParticleReset()
	ParticleType("plain")
	ParticleRadius(radius)
	ParticleAlpha(alpha, 0.1, "constant", 0.1 / life, 0.5)	-- Ramp up fast, ramp down after 50%
	ParticleGravity(gravity, 0.6 * randFloat(0.7, 1.3))				-- Slightly randomized gravity looks better
	ParticleDrag(drag)
	ParticleTile(0)
	ParticleCollide(0)
    ParticleEmissive(0.4, 0.2)
	
	--Emit particles
	for i=1, count do
        local modif = randFloat(-0.02, 0.15)
        local red = 0.05 + modif
        local green = 0.05 + modif
        local blue = 0.05 + modif
        ParticleColor(red, green, blue)			-- Animating color towards white
		p = VecAdd(pos, randVec(2 * radius))
		local v = randVec(randFloat(0.5, 7))
        if big then
            v = VecScale(v, 2)
        end
        if v[2] < 0 then
            v[2] = -v[2]
        end
	
		--Randomize lifetime
		local l = randFloat(life * 0.8, life * 1.2)

		SpawnParticle(p, v, l)
	end
end